using namespace std;

#include <iostream>
#include <unordered_set>
#include <unordered_map>
#include "testlib.h"

typedef long long ll;
typedef vector<vector<int>> graph;

int max_n = 100000;
int max_m = 200000;
ll max_b = 1000'000'000LL;
int max_w_i = 10000;
ll max_c_i = 1000'000'000LL;

ll MAX = 1000'000'000'000'000'000LL;

ll add_checked(const ll &a, ll b, const string &s) {
    ensuref(a <= MAX - b, "%lld + %lld = OVERFLOW, %s", a, b, s.c_str());
    return a + b;
}

ll mul_checked(const ll &a, ll b, const string &s) {
    ensuref(a <= MAX / b, "%lld * %lld = OVERFLOW, %s", a, b, s.c_str());
    return a * b;
}

int tree_max_deg(const graph &g, vector<bool> &visit, int v, int p) {
    visit[v] = true;
    int max_deg = 0;
    for (int u : g[v]) {
        if (u == p) { continue; }
        if (visit[u]) { return 0; }
        int d = tree_max_deg(g, visit, u, v);
        if (d == 0) { return 0; }
        max_deg = max(max_deg, d);
    }
    return max(max_deg, (int) g[v].size());
}

int tree_max_deg(const graph &g) {
    vector<bool> visit(g.size(), false);
    int d = tree_max_deg(g, visit, 1, 0);
    if (d == 0) { return 0; }
    for (int i = 1; i < g.size(); ++i) {
        if (!visit[i]) {
            return 0;
        }
    }
    return d;
}

int main(int argc, char *argv[]) {
    registerValidation(argc, argv);

    int n = inf.readInt(2, max_n, "n");
    inf.readSpace();
    int m = inf.readInt(1, max_m, "m");
    inf.readSpace();
    ll b = inf.readInt(1LL, max_b, "b");
    inf.readEoln();

    vector <ll> ws;
    ll wsum = 0;
    graph g(n + 1);
    for (int i = 0; i < n - 1; ++i) {
        int u = inf.readInt(1, n, "u_i");
        inf.readSpace();
        int v = inf.readInt(1, n, "v_i");
        inf.readSpace();
        ll w = inf.readInt(0, max_w_i, "w_i");
        inf.readEoln();

        ensuref(u != v, "u_i must be not equal v_i, edge %d", i + 1);
        wsum = add_checked(wsum, w, "wsum += w_i");
        ws.push_back(w);
        g[v].push_back(u);
        g[u].push_back(v);
    }

    ll csum = 0;
    vector <ll> cs;
    for (int i = 0; i < m; ++i) {
        int x = inf.readInt(1, n, "x_i");
        inf.readSpace();
        int y = inf.readInt(1, n, "y_i");
        ensuref(x != y, "x_i must be not equal y_i, order number %d", i + 1);

        inf.readSpace();
        ll c = inf.readInt(1LL, max_c_i, "c_i");
        inf.readEoln();

        csum = add_checked(csum, c / b + 1, "csum += c");
        cs.push_back(c);
    }
    inf.readEof();

    int max_deg = tree_max_deg(g);
    ensuref(max_deg > 0, "it isn't tree");
    //    mul_checked(wsum, csum, "wsum * csum");
    if (validator.group() == "1") {
        ensuref(n <= 3 && m <= 3, "g1: n, m must be <= 3");
        ensuref(b <= 3, "g1: b must be <= 3");
        ensuref(*max_element(cs.begin(), cs.end()) <= 3, "g1: c must be <= 3");
    } else if (validator.group() == "2") {
        ensuref(n <= 50 && m <= 50, "g2: n, m must be <= 50");
        ensuref(b <= 50, "g2: b must be <= 50");
        ensuref(*max_element(cs.begin(), cs.end()) <= 50, "g2: c must be <= 50");
    } else if (validator.group() == "3") {
        ensuref(n <= 10000, "g3: n, m must be <= 10^4");
        ensuref(*max_element(ws.begin(), ws.end()) <= 10000, "g3: w_i must be <= 50");
        ensuref(b == max_b, "g3: b must be 10^9");
    } else if (validator.group() == "4") {
        ensuref(max_deg <= 2, "g4: tree must has degs <= 2");
    } else if (validator.group() == "5") {
        ensuref(n <= 500 && m <= 500, "g5: n, m must be <= 500");
    } else if (validator.group() == "6") {
        ensuref(n <= 5000, "g6: n must be <= 5000");
    }
    
    return 0;
}
